/* ----------------------------------------------------------------------------
 * Copyright (c) 2018 Semiconductor Components Industries, LLC (d/b/a
 * ON Semiconductor), All Rights Reserved
 *
 * This code is the property of ON Semiconductor and may not be redistributed
 * in any form without prior written permission from ON Semiconductor.
 * The terms of use and warranty for this code are covered by contractual
 * agreements between ON Semiconductor and the licensee.
 *
 * This is Reusable Code.
 *
 * ----------------------------------------------------------------------------
 * SPI_RSLxx.c
 * - CMSIS-Driver implementation for RSLxx SPI
 * ----------------------------------------------------------------------------
 * $Revision: 1.9 $
 * $Date: 2018/03/14 17:26:54 $
 * ------------------------------------------------------------------------- */
#include "SPI_RSLxx.h"

#define ARM_SPI_DRV_VERSION    ARM_DRIVER_VERSION_MAJOR_MINOR(1, 0)

/* Driver Version */
static const ARM_DRIVER_VERSION DriverVersion =
{
    ARM_SPI_API_VERSION,
    ARM_SPI_DRV_VERSION
};

/* Driver Capabilities */
static const ARM_SPI_CAPABILITIES DriverCapabilities =
{
    0,    /* Simplex Mode (Master and Slave) */
    0,    /* TI Synchronous Serial Interface */
    0,    /* Microwire Interface */
    0    /* Signal Mode Fault event */
};

#if RTE_SPI0

/* SPI0 Run-Time Information */
static SPI_INFO SPI0_Info         = { 0U };
static SPI_TRANSFER_INFO SPI0_TransferInfo = { 0U };
static SPI_PINS SPI0_Pins         = {
    RTE_SPI0_MOSI_PIN,
    RTE_SPI0_MISO_PIN,
    RTE_SPI0_SSEL_PIN,
    RTE_SPI0_SCLK_PIN
};

/* SPI0 Resources */
static const SPI_RESOURCES SPI0_Resources =
{
    (SPI1_Type *)SPI0,
    &SPI0_Pins,
    SPI0_RX_IRQn,
    SPI0_TX_IRQn,
    &SPI0_Info,
    &SPI0_TransferInfo
};
#endif    /* RTE_SPI0 */

#if RTE_SPI1

/* SPI1 Run-Time Information */
static SPI_INFO SPI1_Info         = { 0U };
static SPI_TRANSFER_INFO SPI1_TransferInfo = { 0U };
static SPI_PINS SPI1_Pins         = {
    RTE_SPI1_MOSI_PIN,
    RTE_SPI1_MISO_PIN,
    RTE_SPI1_SSEL_PIN,
    RTE_SPI1_SCLK_PIN
};

/* SPI1 Resources */
static const SPI_RESOURCES SPI1_Resources =
{
    (SPI1_Type *)SPI1,
    &SPI1_Pins,
    SPI1_RX_IRQn,
    SPI1_TX_IRQn,
    &SPI1_Info,
    &SPI1_TransferInfo
};
#endif    /* RTE_SPI1 */

/* ----------------------------------------------------------------------------
 * Function      : void SPI_PrepareNextTXData(SPI_RESOURCES* spi)
 * ----------------------------------------------------------------------------
 * Description   : Support function internal to the driver. Called by
 *                 SPI_Transfer at the start of a transfer and by
 *                 SPI_TX_IRQ_Handler for each new data transfer. This function
 *                 prepares the data for transfer by converting the byte array
 *                 into the configured word size (8, 16, 24 or 32).
 * Inputs        : spi              - Pointer to SPI resources
 * Outputs       : TX_Data ready for next transfer. If tx_buf is NULL, returns
 * Assumptions   : none
 * ------------------------------------------------------------------------- */
static uint32_t SPI_PrepareNextTXData(const SPI_RESOURCES *spi)
{
    uint32_t data = spi->xfer->def_val;
    uint8_t i, ws;

    if (spi->xfer->tx_buf)
    {
        /* Get word size configuration (in bytes) */
        ws = ((spi->reg->CTRL1 & SPI0_CTRL1_SPI0_WORD_SIZE_Mask) + 1) / 8;

        /* Convert byte array into 32bit integer */
        for (i = 0, data = 0; i < ws; i++)
        {
            data |= spi->xfer->tx_buf[i] << (i * 8);
        }

        spi->xfer->tx_buf += ws;
    }

    return data;
}

/* ----------------------------------------------------------------------------
 * Function      : void SPI_TransferCompleted(SPI_RESOURCES* spi)
 * ----------------------------------------------------------------------------
 * Description   : Support function internal to the driver. Called after a
 *                 transfer is completed. Clear flags and set operation to idle
 * Inputs        : spi              - Pointer to SPI resources
 * Outputs       : none
 * Assumptions   : none
 * ------------------------------------------------------------------------- */
static void SPI_TransferCompleted(const SPI_RESOURCES *spi)
{
    /* Clear busy flag */
    spi->info->status.busy = 0U;

    /* Deactivate SSEL line, if in master mode/SS_HW_OUT */
    if ((spi->info->mode & ARM_SPI_MODE_MASTER) &&
        (spi->info->mode & ARM_SPI_SS_MASTER_HW_OUTPUT))
    {
        spi->reg->CTRL1 = (spi->reg->CTRL1 & ~SPI0_CS_1) | SPI0_CS_1;
    }

    /* Stop transfer and set RW operation to Idle */
    spi->reg->CTRL1 &= (~SPI0_START & ~SPI0_CTRL1_SPI0_RW_CMD_Mask);
}

/* ----------------------------------------------------------------------------
 * Function      : void SPI_CalculatePrescale(uint32_t bus_speed,
 *                                            SPI_RESOURCES* spi)
 * ----------------------------------------------------------------------------
 * Description   : SPI clock is a divided form of System Clock. This function
 *                 calculates the SPI prescaler according to the desired bus
 *                 speed and current System Clock.
 * Inputs        : bus_speed              - desired bus speed in bps
 *                 system_clock           - current system clock
 * Outputs       : An integer in the range [0,9]
 * Assumptions   : The system clock can be divided into the required bus speed.
 *                 For precise speeds, make sure: system_clock = bus_speed << x
 *                 Where x is in the range [1,10]
 * ------------------------------------------------------------------------- */
static uint8_t SPI_CalculatePrescale(uint32_t bus_speed, uint32_t system_clock)
{
    uint8_t spi_prescale;

    /* SPI Clock is sourced from SysClk. Find closest prescale value */
    for (spi_prescale = 0U; spi_prescale < SPI_MAX_PRESCALE; spi_prescale++)
    {
        if (bus_speed >= (system_clock >> (spi_prescale + 1U)))
        {
            break;
        }
    }
    return spi_prescale;
}

/* ----------------------------------------------------------------------------
 * Function      : void SPI_TX_IRQHandler(const SPI_RESOURCES *spi)
 * ----------------------------------------------------------------------------
 * Description   : Called by hardware ISR when a byte is transmitted. Place
 *                 transmit data into the SPI, sequentially, byte per byte.
 *                 Manage flags and counters. Application is notified via
 *                 callback function once the transmission is complete
 * Inputs        : spi    - Pointer to SPI resources
 * Outputs       : None
 * Assumptions   : None
 * ------------------------------------------------------------------------- */
static void SPI_TX_IRQHandler(const SPI_RESOURCES *spi)
{
    uint32_t event = 0U;

    if (spi->info->status.busy)
    {
        /* If data lost occurred on last transmission */
        if ((spi->reg->STATUS & SPI0_OVERRUN_TRUE)  ||
            (spi->reg->STATUS & SPI0_UNDERRUN_TRUE))
        {
            spi->info->status.data_lost = 1U;
            event |= ARM_SPI_EVENT_DATA_LOST;

            /* Clear error status */
            spi->reg->STATUS |= SPI0_UNDERRUN_CLEAR | SPI0_OVERRUN_CLEAR;
        }

        spi->xfer->tx_cnt++;

        /* If has more data to transfer */
        if (spi->xfer->tx_cnt < spi->xfer->num)    /* Transfer next byte */
        {
            /* Fill TX_DATA SPI buffer */
            spi->reg->TX_DATA = SPI_PrepareNextTXData(spi);

            /* Start next transfer (if master) */
            spi->reg->CTRL1 |= SPI0_START;
        }
        else if (spi->xfer->tx_cnt == spi->xfer->rx_cnt)
        {
            /* Clear flags and stop transfer. */
            SPI_TransferCompleted(spi);

            event |= ARM_SPI_EVENT_TRANSFER_COMPLETE;
        }
    }

    /* If an event happened and the application registered a callback */
    if (event && spi->info->cb_event)
    {
        spi->info->cb_event(event);
    }
}

/* ----------------------------------------------------------------------------
 * Function      : void SPI_RX_IRQHandler(const SPI_RESOURCES *spi)
 * ----------------------------------------------------------------------------
 * Description   : Called by hardware ISR when a byte is received. Store
 *                 received data in the buffer, manage flags and counters.
 *                 Application is notified via callback function once the
 *                 receive is complete
 * Inputs        : spi    - Pointer to SPI resources
 * Outputs       : None
 * Assumptions   : None
 * ------------------------------------------------------------------------- */
static void SPI_RX_IRQHandler(const SPI_RESOURCES *spi)
{
    uint32_t data, i, ws;
    uint32_t event = 0U;

    if (spi->info->status.busy && spi->xfer->rx_cnt < spi->xfer->num)
    {
        /* If data lost occurred on last transmission */
        if ((spi->reg->STATUS & SPI0_OVERRUN_TRUE)  ||
            (spi->reg->STATUS & SPI0_UNDERRUN_TRUE))
        {
            spi->info->status.data_lost = 1U;
            event |= ARM_SPI_EVENT_DATA_LOST;

            /* Clear error status */
            spi->reg->STATUS |= SPI0_UNDERRUN_CLEAR | SPI0_OVERRUN_CLEAR;
        }

        /* Read received data from SPI RX_DATA register */
        data = spi->reg->RX_DATA;

        if (spi->xfer->rx_buf != NULL)
        {
            /* Get word size configuration (in bytes) */
            ws = ((spi->reg->CTRL1 & SPI0_CTRL1_SPI0_WORD_SIZE_Mask) + 1) / 8;

            /* Split received data into bytes and store into rx_buf */
            for (i = 0; i < ws; i++)
            {
                *(spi->xfer->rx_buf++) = (uint8_t)(data >> (i * 8));
            }
        }

        spi->xfer->rx_cnt++;

        /* If transfer has finished */
        if ((spi->xfer->rx_cnt == spi->xfer->num) &&
            (spi->xfer->tx_cnt == spi->xfer->rx_cnt))
        {
            /* Clear flags and stop transfer. */
            SPI_TransferCompleted(spi);

            event |= ARM_SPI_EVENT_TRANSFER_COMPLETE;
        }
    }

    /* If an event happened and the application registered a callback */
    if (event && spi->info->cb_event)
    {
        spi->info->cb_event(event);
    }
}

/* ----------------------------------------------------------------------------
 * Function      : void SPIx_GetVersion(void)
 * ----------------------------------------------------------------------------
 * Description   : Driver version
 * Inputs        : None
 * Outputs       : ARM_DRIVER_VERSION
 * Assumptions   : None
 * ------------------------------------------------------------------------- */
static ARM_DRIVER_VERSION SPIx_GetVersion(void)
{
    return DriverVersion;
}

/* ----------------------------------------------------------------------------
 * Function      : void SPIx_GetCapabilities(void)
 * ----------------------------------------------------------------------------
 * Description   : Get SPIx driver capabilities
 * Inputs        : None
 * Outputs       : ARM_SPI_CAPABILITIES
 * Assumptions   : None
 * ------------------------------------------------------------------------- */
static ARM_SPI_CAPABILITIES SPIx_GetCapabilities(void)
{
    return DriverCapabilities;
}

/* ----------------------------------------------------------------------------
 * Function      : void SPI_Initialize(ARM_SPI_SignalEvent_t cb_event,
 *                                     const SPI_RESOURCES* spi)
 * ----------------------------------------------------------------------------
 * Description   : Initialize SPI flags, GPIO pins and reference to
 *                 callback function.
 * Inputs        : cb_event - pointer to callback function (optional)
 *                 spi    - Pointer to SPI resources
 * Outputs       : ARM_DRIVER_OK
 * Assumptions   : None
 * ------------------------------------------------------------------------- */
static int32_t SPI_Initialize(ARM_SPI_SignalEvent_t cb_event,
                              const SPI_RESOURCES *spi)
{
    if (spi->info->state & SPI_INITIALIZED)
    {
        /* Driver is already initialized */
        return ARM_DRIVER_OK;
    }

    /* Initialize SPI Run-Time Resources */
    spi->info->cb_event          = cb_event;
    spi->info->status.busy       = 0U;
    spi->info->status.data_lost  = 0U;
    spi->info->status.mode_fault = 0U;

    /* Clear transfer information */
    memset(spi->xfer, 0, sizeof(SPI_TRANSFER_INFO));

    spi->info->state = SPI_INITIALIZED;

    return ARM_DRIVER_OK;
}

/* ----------------------------------------------------------------------------
 * Function      : void SPI_Uninitialize(const SPI_RESOURCES *spi)
 * ----------------------------------------------------------------------------
 * Description   : Uninitialize SPI flags, GPIO pins and removes reference to
 *                 callback function.
 * Inputs        : spi - Pointer to SPI resources
 * Outputs       : ARM_DRIVER_OK
 * Assumptions   : None
 * ------------------------------------------------------------------------- */
static int32_t SPI_Uninitialize(const SPI_RESOURCES *spi)
{
    /* Reset pin configuration */
    DIO->CFG[spi->pins->miso] = DIO_MODE_DISABLE;
    DIO->CFG[spi->pins->mosi] = DIO_MODE_DISABLE;
    DIO->CFG[spi->pins->ssel] = DIO_MODE_DISABLE;
    DIO->CFG[spi->pins->sclk] = DIO_MODE_DISABLE;

    /* Reset DIO_SPI_SRC configuration */
    DIO->SPI_SRC[(spi->reg == SPI1)] = (SPI_SERI_SRC_CONST_HIGH |
                                        SPI_CS_SRC_CONST_HIGH   |
                                        SPI_CLK_SRC_CONST_HIGH);

    /* Clear SPI state */
    spi->info->state = 0U;

    /* Remove reference to callback function */
    spi->info->cb_event = NULL;

    return ARM_DRIVER_OK;
}

/* ----------------------------------------------------------------------------
 * Function      : void SPI_PowerControl(ARM_POWER_STATE state,
 *                                       const SPI_RESOURCES *spi)
 * ----------------------------------------------------------------------------
 * Description   : Operates the power modes of the SPI interface
 * Inputs        : state - ARM_POWER_FULL or ARM_POWER_OFF
 *                 spi - Pointer to SPI resources
 * Outputs       : ARM_DRIVER_OK - if the operation is successful
 *                 ARM_DRIVER_ERROR_UNSUPPORTED - if argument is ARM_POWER_LOW
 *                                                or an invalid value
 * Assumptions   : None
 * ------------------------------------------------------------------------- */
static int32_t SPI_PowerControl(ARM_POWER_STATE state,
                                const SPI_RESOURCES *spi)
{
    switch (state)
    {
        case ARM_POWER_OFF:
        {
            /* Disable SPI interrupts */
            NVIC_DisableIRQ(spi->rx_irqn);
            NVIC_DisableIRQ(spi->tx_irqn);

            /* Clear pending SPI interrupts in NVIC */
            NVIC_ClearPendingIRQ(spi->rx_irqn);
            NVIC_ClearPendingIRQ(spi->tx_irqn);

            /* Disable SPI */
            spi->reg->CTRL0 = SPI0_DISABLE;

            /* Clear status flags */
            spi->info->status.busy       = 0U;
            spi->info->status.data_lost  = 0U;
            spi->info->status.mode_fault = 0U;

            spi->info->state &= ~SPI_POWERED;
        }
        break;

        case ARM_POWER_FULL:
        {
            if ((spi->info->state & SPI_INITIALIZED) == 0U)
            {
                return ARM_DRIVER_ERROR;
            }
            if ((spi->info->state & SPI_POWERED) != 0U)
            {
                return ARM_DRIVER_OK;
            }

            /* Clear Status flags */
            spi->info->status.busy       = 0U;
            spi->info->status.data_lost  = 0U;
            spi->info->status.mode_fault = 0U;

            spi->xfer->def_val           = 0U;

            /* Enable RX/TX/ERR interrupts */
            NVIC_ClearPendingIRQ(spi->tx_irqn);
            NVIC_ClearPendingIRQ(spi->rx_irqn);
            NVIC_EnableIRQ(spi->tx_irqn);
            NVIC_EnableIRQ(spi->rx_irqn);

            /* Set powered flag */
            spi->info->state |= SPI_POWERED;
        }
        break;

        case ARM_POWER_LOW:
        default:
            return ARM_DRIVER_ERROR_UNSUPPORTED;
    }

    return ARM_DRIVER_OK;
}

/* ----------------------------------------------------------------------------
 * Function      : void SPI_Control(uint32_t control, uint32_t arg,
 *                                  const SPI_RESOURCES *spi)
 * ----------------------------------------------------------------------------
 * Description   : Control SPI Interface
 * Inputs        : control - Control operation
 *                 arg     - Argument of operation (optional)
 *                 spi     - Pointer to SPI resources
 * Outputs       : Execution status
 * Assumptions   : None
 * ------------------------------------------------------------------------- */
static int32_t SPI_Control(uint32_t control, uint32_t arg,
                           const SPI_RESOURCES *spi)
{
    uint32_t mode, ctrl0, ctrl1, spi_prescale;
    mode  = 0U;
    ctrl0 = (SPI0_OVERRUN_INT_DISABLE |
             SPI0_UNDERRUN_INT_DISABLE |
             SPI0_CONTROLLER_CM3       |
             SPI0_MODE_SELECT_MANUAL   |
             SPI0_ENABLE);

    ctrl1 = SPI0_IDLE | SPI0_NOP | SPI0_CS_1;

    if ((spi->info->state & SPI_POWERED) == 0U)
    {
        /* SPI not powered */
        return ARM_DRIVER_ERROR;
    }

    /* Abort transfer */
    if ((control & ARM_SPI_CONTROL_Msk) == ARM_SPI_ABORT_TRANSFER)
    {
        /* Stop transfer and set RW operation to Idle */
        spi->reg->CTRL1 &= (~SPI0_START & ~SPI0_CTRL1_SPI0_RW_CMD_Mask);

        /* Clear transfer information and status */
        memset(spi->xfer, 0, sizeof(SPI_TRANSFER_INFO));
        spi->info->status.busy = 0U;
        return ARM_DRIVER_OK;
    }

    /* Check for busy flag */
    if (spi->info->status.busy)
    {
        return ARM_DRIVER_ERROR_BUSY;
    }

    switch (control & ARM_SPI_CONTROL_Msk)
    {
        case ARM_SPI_MODE_INACTIVE:
        {
            /* Disable SPI */
            spi->reg->CTRL0   = 0U;
            spi->info->state &= ~SPI_CONFIGURED;
            spi->info->mode   = ((spi->info->mode & ~ARM_SPI_CONTROL_Msk) |
                                 ARM_SPI_MODE_INACTIVE);
            return ARM_DRIVER_OK;
        }

        case ARM_SPI_MODE_MASTER:
        {
            /* Set master mode */
            mode  |= ARM_SPI_MODE_MASTER;
            ctrl0 |= SPI0_SELECT_MASTER;

            /* Configure GPIO */
            Sys_SPI_DIOConfig((spi->reg == SPI1), SPI0_SELECT_MASTER,
                              (DIO_6X_DRIVE | DIO_LPF_DISABLE |
                               DIO_WEAK_PULL_UP),
                              spi->pins->sclk, spi->pins->ssel,
                              spi->pins->miso, spi->pins->mosi);

            switch (control & ARM_SPI_SS_MASTER_MODE_Msk)
            {
                case ARM_SPI_SS_MASTER_SW:
                case ARM_SPI_SS_MASTER_HW_OUTPUT:
                {
                    /* SSEL pin already configured above */
                    mode |= (control & ARM_SPI_SS_MASTER_MODE_Msk);
                }
                break;

                case ARM_SPI_SS_MASTER_UNUSED:
                {
                    /* Unconfigure SSEL Pin */
                    Sys_DIO_Config(spi->pins->ssel,  DIO_MODE_DISABLE);
                    mode |= ARM_SPI_SS_MASTER_UNUSED;
                }
                break;

                case ARM_SPI_SS_MASTER_HW_INPUT:
                default:
                    return ARM_SPI_ERROR_SS_MODE;
            }
        }
        break;

        case ARM_SPI_MODE_SLAVE:
        {
            /* Set slave mode */
            mode  |= ARM_SPI_MODE_SLAVE;
            ctrl0 |= SPI0_SELECT_SLAVE;

            /* Configure GPIO. MOSI and MISO switched. */
            Sys_SPI_DIOConfig((spi->reg == SPI1), SPI0_SELECT_SLAVE,
                              (DIO_6X_DRIVE | DIO_LPF_DISABLE |
                               DIO_WEAK_PULL_UP),
                              spi->pins->sclk, spi->pins->ssel,
                              spi->pins->mosi, spi->pins->miso);

            switch (control & ARM_SPI_SS_SLAVE_MODE_Msk)
            {
                case ARM_SPI_SS_SLAVE_HW:
                {
                    /* Already configured above */
                }
                break;

                case ARM_SPI_SS_SLAVE_SW:
                {
                    /* Unconfigure SSEL Pin */
                    Sys_DIO_Config(spi->pins->ssel,  DIO_MODE_DISABLE);
                }
                break;

                default:
                    return ARM_SPI_ERROR_SS_MODE;
            }
        }
        break;

        /* Simplex mode not supported */
        case ARM_SPI_MODE_MASTER_SIMPLEX:
        case ARM_SPI_MODE_SLAVE_SIMPLEX:
        {
            return ARM_SPI_ERROR_MODE;
        }

        case ARM_SPI_SET_BUS_SPEED:
        {
            /* If system clock is not fast enough */
            if (SystemCoreClock < 4 * arg)
            {
                return ARM_DRIVER_ERROR_UNSUPPORTED;
            }

            /* Set SPI prescale */
            spi_prescale = SPI_CalculatePrescale(arg, SystemCoreClock);
            spi->reg->CTRL0 = ((spi->reg->CTRL0 &
                                ~SPI0_CTRL0_SPI0_PRESCALE_Mask) |
                               spi_prescale);
            return ARM_DRIVER_OK;
        }

        case ARM_SPI_GET_BUS_SPEED:
        {
            /* Read SPI prescale value */
            spi_prescale = ((spi->reg->CTRL0 >> SPI0_CTRL0_SPI0_PRESCALE_Pos) &
                            SPI0_CTRL0_SPI0_PRESCALE_Mask);

            /* Return current bus speed */
            return (SystemCoreClock >> (spi_prescale + 1U));
        }

        case ARM_SPI_SET_DEFAULT_TX_VALUE:
        {
            spi->xfer->def_val = (uint16_t)(arg & 0xFFFFU);
            return ARM_DRIVER_OK;
        }

        case ARM_SPI_CONTROL_SS:
        {
            if (((spi->info->mode & ARM_SPI_CONTROL_Msk) !=
                 ARM_SPI_MODE_MASTER) ||
                ((spi->info->mode & ARM_SPI_SS_MASTER_MODE_Msk) !=
                 ARM_SPI_SS_MASTER_SW))
            {
                /* SSEL line can only be controlled in master/SW mode */
                return ARM_DRIVER_ERROR;
            }

            if (arg == ARM_SPI_SS_ACTIVE)
            {
                /* Set SSEL pin low */
                spi->reg->CTRL1 = (spi->reg->CTRL1 & ~SPI0_CS_1) | SPI0_CS_0;
            }
            else
            {
                /* Set SSEL pin high */
                spi->reg->CTRL1 = (spi->reg->CTRL1 & ~SPI0_CS_1) | SPI0_CS_1;
            }
            return ARM_DRIVER_OK;
        }

        default:
            return ARM_DRIVER_ERROR_UNSUPPORTED;
    }

    /* Frame format */
    switch (control & ARM_SPI_FRAME_FORMAT_Msk)
    {
        case ARM_SPI_CPOL0_CPHA0:
        {
            ctrl0 |= SPI0_CLK_POLARITY_NORMAL;
        }
        break;

        case ARM_SPI_CPOL1_CPHA0:
        {
            ctrl0 |= SPI0_CLK_POLARITY_INVERSE;
        }
        break;

        case ARM_SPI_CPOL1_CPHA1:
        case ARM_SPI_CPOL0_CPHA1:
        case ARM_SPI_TI_SSI:
        case ARM_SPI_MICROWIRE:
        default:
            return ARM_SPI_ERROR_FRAME_FORMAT;
    }

    /* Data Bits */
    switch (control & ARM_SPI_DATA_BITS_Msk)
    {
        case ARM_SPI_DATA_BITS(8U):
        {
            ctrl1 |= SPI0_WORD_SIZE_8;
        }
        break;

        case ARM_SPI_DATA_BITS(16U):
        {
            ctrl1 |= SPI0_WORD_SIZE_16;
        }
        break;

        case ARM_SPI_DATA_BITS(24U):
        {
            ctrl1 |= SPI0_WORD_SIZE_24;
        }
        break;

        case ARM_SPI_DATA_BITS(32U):
        {
            ctrl1 |= SPI0_WORD_SIZE_32;
        }
        break;

        default:
            return ARM_SPI_ERROR_DATA_BITS;
    }

    /* Bit order supports MSB to LSB only */
    if ((control & ARM_SPI_BIT_ORDER_Msk) != ARM_SPI_MSB_LSB)
    {
        return ARM_SPI_ERROR_BIT_ORDER;
    }

    /* Set SPI Bus Speed */
    spi_prescale = SPI_CalculatePrescale(arg, SystemCoreClock);
    ctrl0 |= (spi_prescale << SPI0_CTRL0_SPI0_PRESCALE_Pos);

    /* Save mode */
    spi->info->mode = mode;

    /* Set SPI registers */
    Sys_SPI_Config((spi->reg == SPI1), ctrl0);
    Sys_SPI_TransferConfig((spi->reg == SPI1), ctrl1);

    spi->info->state |= SPI_CONFIGURED;

    return ARM_DRIVER_OK;
}

/* ----------------------------------------------------------------------------
 * Function      : int32_t SPI_Transfer(const void *data_out, void *data_in,
 *                                      uint32_t num, const SPI_RESOURCES *spi)
 * ----------------------------------------------------------------------------
 * Description   : Non-blocking transfer function. If a callback was previously
 *                 registered using the Initialize function, the caller is
 *                 notified when 'num' bytes are transferred.
 * Inputs        : data_out              - pointer to tx buffer
 *                 data_in               - pointer to rx buffer
 *                 num                   - number of bytes to send
 *                 spi                   - Pointer to SPI resources
 * Outputs       : ARM_DRIVER_OK         - if it can successfully start the
 *                                         operation
 *                 ARM_DRIVER_ERROR_BUSY - if it's already busy handling
 *                                         a previous transfer
 * Assumptions   : The data buffers point to a memory area with 'num'
 *                 bytes of size
 * ------------------------------------------------------------------------- */
static int32_t SPI_Transfer(const void *data_out, void *data_in, uint32_t num,
                            const SPI_RESOURCES *spi)
{
    if (((data_out == NULL) && (data_in == NULL)) || (num == 0U))
    {
        /* Invalid parameters */
        return ARM_DRIVER_ERROR_PARAMETER;
    }

    if ((spi->info->state & SPI_CONFIGURED) == 0U)
    {
        /* SPI is not configured */
        return ARM_DRIVER_ERROR;
    }

    if (spi->info->status.busy)
    {
        /* Previous send is not completed yet */
        return ARM_DRIVER_ERROR_BUSY;
    }

    /* Updates statuses */
    spi->info->status.busy       = 1U;
    spi->info->status.data_lost  = 0U;
    spi->info->status.mode_fault = 0U;

    /* Save transmit buffer info */
    spi->xfer->rx_buf = (uint8_t *)data_in;
    spi->xfer->tx_buf = (uint8_t *)data_out;
    spi->xfer->num    = num;
    spi->xfer->rx_cnt = 0U;
    spi->xfer->tx_cnt = 0U;

    /* Activate SSEL line, if HW controlled */
    if ((spi->info->mode & ARM_SPI_MODE_MASTER) &&
        (spi->info->mode & ARM_SPI_SS_MASTER_HW_OUTPUT))
    {
        spi->reg->CTRL1 &= ~SPI0_CS_1;
    }

    /* Fill SPI TX_DATA buffer for first transmission.
     * Remaining transmissions will occur via the SPI Interrupts. */
    spi->reg->TX_DATA = SPI_PrepareNextTXData(spi);

    /* Configure SPI to RW operation and start transfer (master) */
    spi->reg->CTRL1 |= (SPI0_START | SPI0_RW_DATA);

    return ARM_DRIVER_OK;
}

/* ----------------------------------------------------------------------------
 * Function      : void SPI_GetDataCount(const SPI_RESOURCES *spi)
 * ----------------------------------------------------------------------------
 * Description   : Get SPIx data count. This count is reset for every receive
 *                 operation.
 * Inputs        : spi    - Pointer to SPI resources
 * Outputs       : Number of bytes received by SPIx
 * Assumptions   : None
 * ------------------------------------------------------------------------- */
static uint32_t SPI_GetDataCount(const SPI_RESOURCES *spi)
{
    return spi->xfer->rx_cnt;
}

/* ----------------------------------------------------------------------------
 * Function      : void SPI_GetStatus(const SPI_RESOURCES *spi)
 * ----------------------------------------------------------------------------
 * Description   : Get SPIx status
 * Inputs        : spi    - Pointer to SPI resources
 * Outputs       : Return SPIx status as an ARM_SPI_STATUS structure
 * Assumptions   : None
 * ------------------------------------------------------------------------- */
static ARM_SPI_STATUS SPI_GetStatus(const SPI_RESOURCES *spi)
{
    ARM_SPI_STATUS status;

    status.busy       = spi->info->status.busy;
    status.data_lost  = spi->info->status.data_lost;
    status.mode_fault = 0U;

    return status;
}

/* SPI0 Driver Wrapper functions. See functions above for usage */
#if RTE_SPI0
static int32_t SPI0_Initialize(ARM_SPI_SignalEvent_t cb_event)
{
    return SPI_Initialize(cb_event, &SPI0_Resources);
}

static int32_t SPI0_Uninitialize(void)
{
    return SPI_Uninitialize(&SPI0_Resources);
}

static int32_t SPI0_PowerControl(ARM_POWER_STATE state)
{
    return SPI_PowerControl(state, &SPI0_Resources);
}

static int32_t SPI0_Send(const void *data, uint32_t num)
{
    return SPI_Transfer(data, NULL, num, &SPI0_Resources);
}

static int32_t SPI0_Receive(void *data, uint32_t num)
{
    return SPI_Transfer(NULL, data, num, &SPI0_Resources);
}

static int32_t SPI0_Transfer(const void *data_out, void *data_in, uint32_t num)
{
    return SPI_Transfer(data_out, data_in, num, &SPI0_Resources);
}

static uint32_t SPI0_GetDataCount(void)
{
    return SPI_GetDataCount(&SPI0_Resources);
}

static int32_t SPI0_Control(uint32_t control, uint32_t arg)
{
    return SPI_Control(control, arg, &SPI0_Resources);
}

static ARM_SPI_STATUS SPI0_GetStatus(void)
{
    return SPI_GetStatus(&SPI0_Resources);
}

void SPI0_TX_IRQHandler(void)
{
    SPI_TX_IRQHandler(&SPI0_Resources);
}

void SPI0_RX_IRQHandler(void)
{
    SPI_RX_IRQHandler(&SPI0_Resources);
}

/* SPI0 Driver Control Block */
ARM_DRIVER_SPI Driver_SPI0 =
{
    SPIx_GetVersion,
    SPIx_GetCapabilities,
    SPI0_Initialize,
    SPI0_Uninitialize,
    SPI0_PowerControl,
    SPI0_Send,
    SPI0_Receive,
    SPI0_Transfer,
    SPI0_GetDataCount,
    SPI0_Control,
    SPI0_GetStatus
};
#endif    /* if RTE_SPI0 */

/* SPI1 Driver Wrapper functions. See functions above for usage */
#if RTE_SPI1
static int32_t SPI1_Initialize(ARM_SPI_SignalEvent_t cb_event)
{
    return SPI_Initialize(cb_event, &SPI1_Resources);
}

static int32_t SPI1_Uninitialize(void)
{
    return SPI_Uninitialize(&SPI1_Resources);
}

static int32_t SPI1_PowerControl(ARM_POWER_STATE state)
{
    return SPI_PowerControl(state, &SPI1_Resources);
}

static int32_t SPI1_Send(const void *data, uint32_t num)
{
    return SPI_Transfer(data, NULL, num, &SPI1_Resources);
}

static int32_t SPI1_Receive(void *data, uint32_t num)
{
    return SPI_Transfer(NULL, data, num, &SPI1_Resources);
}

static int32_t SPI1_Transfer(const void *data_out, void *data_in, uint32_t num)
{
    return SPI_Transfer(data_out, data_in, num, &SPI1_Resources);
}

static uint32_t SPI1_GetDataCount(void)
{
    return SPI_GetDataCount(&SPI1_Resources);
}

static int32_t SPI1_Control(uint32_t control, uint32_t arg)
{
    return SPI_Control(control, arg, &SPI1_Resources);
}

static ARM_SPI_STATUS SPI1_GetStatus(void)
{
    return SPI_GetStatus(&SPI1_Resources);
}

void SPI1_TX_IRQHandler(void)
{
    SPI_TX_IRQHandler(&SPI1_Resources);
}

void SPI1_RX_IRQHandler(void)
{
    SPI_RX_IRQHandler(&SPI1_Resources);
}

/* SPI1 Driver Control Block */
ARM_DRIVER_SPI Driver_SPI1 =
{
    SPIx_GetVersion,
    SPIx_GetCapabilities,
    SPI1_Initialize,
    SPI1_Uninitialize,
    SPI1_PowerControl,
    SPI1_Send,
    SPI1_Receive,
    SPI1_Transfer,
    SPI1_GetDataCount,
    SPI1_Control,
    SPI1_GetStatus
};
#endif    /* if RTE_SPI1 */
